@@ -80,7 +80,7 @@
 #define MAX_FINAL_RTR_ADVERTISEMENTS	3
 #define MIN_DELAY_BETWEEN_RAS		3.0
 #define MIN_DELAY_BETWEEN_RAS_MIPv6     (30.0/1000.0)
-#define MAX_RA_DELAY_TIME		(1000.0/2.0)	/* milliseconds */
+#define MAX_RA_DELAY_SECONDS		(0.5)
 
 /* Host constants: */
 
