@@ -16,14 +16,17 @@
 #include "includes.h"
 #include "radvd.h"
 
+__attribute__ ((format(printf, 2, 0)))
+static int vlog(int prio, char const *format, va_list ap);
+
 static int log_method = L_NONE;
-static char *log_ident;
-static char *log_file;
+static char const *log_ident;
+static char const *log_file;
 static FILE *log_file_fd;
 static int log_facility;
 static int debug_level = 0;
 
-int log_open(int method, char *ident, char *log, int facility)
+int log_open(int method, char const *ident, char const *log, int facility)
 {
 	log_method = method;
 	log_ident = ident;
@@ -44,25 +47,26 @@ int log_open(int method, char *ident, char *log, int facility)
 		break;
 	case L_LOGFILE:
 		if (!log) {
-			fprintf(stderr, "%s: no logfile specified\n", log_ident);
-			return (-1);
+			fprintf(stderr, "%s (%d): no logfile specified\n", log_ident, getpid());
+			return -1;
 		}
 		log_file = log;
 		if ((log_file_fd = fopen(log_file, "a")) == NULL) {
-			fprintf(stderr, "%s: can't open %s: %s\n", log_ident, log_file, strerror(errno));
-			return (-1);
+			fprintf(stderr, "%s (%d): can't open %s: %s\n", log_ident, getpid(), log_file, strerror(errno));
+			return -1;
 		}
 		break;
 	default:
-		fprintf(stderr, "%s: unknown logging method: %d\n", log_ident, log_method);
+		fprintf(stderr, "%s (%d): unknown logging method: %d\n", log_ident, getpid(), log_method);
 		log_method = L_NONE;
-		return (-1);
+		return -1;
 	}
 	return 0;
 }
 
 /* note: [dfv]log() is also called from root context */
-static int vlog(int prio, char *format, va_list ap)
+__attribute__ ((format(printf, 2, 0)))
+static int vlog(int prio, char const *format, va_list ap)
 {
 	char tstamp[64], buff[1024];
 	struct tm *tm;
@@ -85,7 +89,7 @@ static int vlog(int prio, char *format, va_list ap)
 		tm = localtime(&current);
 		(void)strftime(tstamp, sizeof(tstamp), LOG_TIME_FORMAT, tm);
 
-		fprintf(stderr, "[%s] %s: %s\n", tstamp, log_ident, buff);
+		fprintf(stderr, "[%s] %s (%d): %s\n", tstamp, log_ident, getpid(), buff);
 		fflush(stderr);
 		break;
 	case L_LOGFILE:
@@ -93,30 +97,29 @@ static int vlog(int prio, char *format, va_list ap)
 		tm = localtime(&current);
 		(void)strftime(tstamp, sizeof(tstamp), LOG_TIME_FORMAT, tm);
 
-		fprintf(log_file_fd, "[%s] %s: %s\n", tstamp, log_ident, buff);
+		fprintf(log_file_fd, "[%s] %s (%d): %s\n", tstamp, log_ident, getpid(), buff);
 		fflush(log_file_fd);
 		break;
 	default:
-		fprintf(stderr, "%s: unknown logging method: %d\n", log_ident, log_method);
+		fprintf(stderr, "%s (%d): unknown logging method: %d\n", log_ident, getpid(), log_method);
 		log_method = L_NONE;
-		return (-1);
+		return -1;
 	}
 	return 0;
 }
 
-void dlog(int prio, int level, char *format, ...)
+void dlog(int prio, int level, char const *format, ...)
 {
-	va_list ap;
-
 	if (debug_level < level)
 		return;
 
+	va_list ap;
 	va_start(ap, format);
 	vlog(prio, format, ap);
 	va_end(ap);
 }
 
-void flog(int prio, char *format, ...)
+void flog(int prio, char const *format, ...)
 {
 	va_list ap;
 
@@ -139,9 +142,9 @@ int log_close(void)
 		fclose(log_file_fd);
 		break;
 	default:
-		fprintf(stderr, "%s: unknown logging method: %d\n", log_ident, log_method);
+		fprintf(stderr, "%s (%d): unknown logging method: %d\n", log_ident, getpid(), log_method);
 		log_method = L_NONE;
-		return (-1);
+		return -1;
 	}
 	return 0;
 }
