@@ -24,39 +24,40 @@
 
 #include <poll.h>
 #include <sys/file.h>
+#include <libgen.h>
 
 #ifdef HAVE_GETOPT_LONG
 
 /* *INDENT-OFF* */
 static char usage_str[] = {
 "\n"
+"  -C, --config=PATH       Set the config file.  Default is /etc/radvd.d.\n"
 "  -c, --configtest        Parse the config file and exit.\n"
-"  -C, --config=PATH       Sets the config file.  Default is /etc/radvd.d.\n"
-"  -d, --debug=NUM         Sets the debug level.  Values can be 1, 2, 3, 4 or 5.\n"
-"  -f, --facility=NUM      Sets the logging facility.\n"
+"  -d, --debug=NUM         Set the debug level.  Values can be 1, 2, 3, 4 or 5.\n"
+"  -f, --facility=NUM      Set the logging facility.\n"
 "  -h, --help              Show this help screen.\n"
-"  -l, --logfile=PATH      Sets the log file.\n"
-"  -m, --logmethod=X       Sets the log method to one of: syslog, stderr, stderr_syslog, logfile, or none.\n"
-"  -p, --pidfile=PATH      Sets the pid file.\n"
+"  -l, --logfile=PATH      Set the log file.\n"
+"  -m, --logmethod=X       Set method to: syslog, stderr, stderr_syslog, logfile, or none.\n"
+"  -n, --nodaemon          Prevent the daemonizing.\n"
+"  -p, --pidfile=PATH      Set the pid file.\n"
 "  -t, --chrootdir=PATH    Chroot to the specified path.\n"
 "  -u, --username=USER     Switch to the specified user.\n"
-"  -n, --nodaemon          Prevent the daemonizing.\n"
 "  -v, --version           Print the version and quit.\n"
 };
 
 static struct option prog_opt[] = {
-	{"debug", 1, 0, 'd'},
-	{"configtest", 0, 0, 'c'},
+	{"chrootdir", 1, 0, 't'},
 	{"config", 1, 0, 'C'},
-	{"pidfile", 1, 0, 'p'},
+	{"configtest", 0, 0, 'c'},
+	{"debug", 1, 0, 'd'},
+	{"facility", 1, 0, 'f'},
+	{"help", 0, 0, 'h'},
 	{"logfile", 1, 0, 'l'},
 	{"logmethod", 1, 0, 'm'},
-	{"facility", 1, 0, 'f'},
-	{"username", 1, 0, 'u'},
-	{"chrootdir", 1, 0, 't'},
-	{"version", 0, 0, 'v'},
-	{"help", 0, 0, 'h'},
 	{"nodaemon", 0, 0, 'n'},
+	{"pidfile", 1, 0, 'p'},
+	{"username", 1, 0, 'u'},
+	{"version", 0, 0, 'v'},
 	{NULL, 0, 0, 0}
 };
 
@@ -72,32 +73,113 @@ static char usage_str[] = {
 #endif
 
 static volatile int sighup_received = 0;
-static volatile int sigterm_received = 0;
 static volatile int sigint_received = 0;
+static volatile int sigterm_received = 0;
 static volatile int sigusr1_received = 0;
 
-static void sighup_handler(int sig);
-static void sigterm_handler(int sig);
-static void sigint_handler(int sig);
-static void sigusr1_handler(int sig);
-static void timer_handler(int sock, struct Interface *iface);
+static int check_conffile_perm(const char *, const char *);
+static int drop_root_privileges(const char *);
+static int open_and_lock_pid_file(char const * daemon_pid_file_ident);
+static int write_pid_file(char const * daemon_pid_file_ident, pid_t pid);
+static pid_t daemonp(int nochdir, int noclose, char const * daemon_pid_file_ident);
+static pid_t do_daemonize(int log_method, char const * daemon_pid_file_ident);
+static struct Interface * main_loop(int sock, struct Interface *ifaces, char const *conf_path);
+static struct Interface *reload_config(int sock, struct Interface *ifaces, char const *conf_path);
+static void check_pid_file(char const * daemon_pid_file_ident);
 static void config_interface(struct Interface *iface);
 static void kickoff_adverts(int sock, struct Interface *iface);
-static void stop_advert_foo(struct Interface *iface, void *data);
-static void stop_adverts(int sock, struct Interface *ifaces);
-static void version(void);
-static void usage(char const *pname);
-static int drop_root_privileges(const char *);
-static int check_conffile_perm(const char *, const char *);
+static void reset_prefix_lifetimes(struct Interface *ifaces);
+static void reset_prefix_lifetimes_foo(struct Interface *iface, void *data);
 static void setup_iface_foo(struct Interface *iface, void *data);
 static void setup_ifaces(int sock, struct Interface *ifaces);
-static void main_loop(int sock, struct Interface *ifaces, char const *conf_path);
-static void reset_prefix_lifetimes_foo(struct Interface *iface, void *data);
-static void reset_prefix_lifetimes(struct Interface *ifaces);
-static struct Interface *reload_config(int sock, struct Interface *ifaces, char const *conf_path);
-static void do_daemonize(int log_method);
-static int open_and_lock_pid_file(char const * daemon_pid_file_ident);
-static int write_pid_file(int pid_fd);
+static void sighup_handler(int sig);
+static void sigint_handler(int sig);
+static void sigterm_handler(int sig);
+static void sigusr1_handler(int sig);
+static void stop_advert_foo(struct Interface *iface, void *data);
+static void stop_adverts(int sock, struct Interface *ifaces);
+static void timer_handler(int sock, struct Interface *iface);
+static void usage(char const *pname);
+static void version(void);
+
+/* daemonize and write pid file.  The pid of the daemon child process
+ * will be written to the pid file from the *parent* process.  This
+ * insures there is no race condition as described in redhat bug 664783. */
+static pid_t daemonp(int nochdir, int noclose, char const * daemon_pid_file_ident)
+{
+	int pipe_ends[2];
+
+	if (0 != pipe(pipe_ends)) {
+		flog(LOG_ERR, "unable to create pipe: %s", strerror(errno));
+		exit(-1);
+	}
+
+	pid_t pid = fork();
+
+	if (-1 == pid) {
+		flog(LOG_ERR, "unable to fork in daemonp");
+		exit(-1);
+	} else if (0 == pid) {
+		/* Child process, detached.. */
+		pid = getpid();
+		close(pipe_ends[0]);
+		if (0 != write_pid_file(daemon_pid_file_ident, pid)) {
+			flog(LOG_ERR, "failure writing pid file");
+			exit(-1);
+		}
+		if (sizeof(pid) != write(pipe_ends[1], &pid, sizeof(pid))) {
+			flog(LOG_ERR, "failure piping pid to parent process");
+		}
+
+		umask(0);
+		if (-1 == setsid()) {
+			flog(LOG_ERR, "unable to become a session leader: %s", strerror(errno));
+			exit(-1);
+		}
+
+		if (nochdir == 0) {
+			if (chdir("/") == -1) {
+				perror("chdir");
+				exit(1);
+			}
+		}
+		if (noclose == 0) {
+			close(STDIN_FILENO);
+			close(STDOUT_FILENO);
+			close(STDERR_FILENO);
+			if (open("/dev/null", O_RDONLY) == -1) {
+				flog(LOG_ERR, "unable to redirect stdin to /dev/null");
+				exit(-1);
+			}
+			if (open("/dev/null", O_WRONLY) == -1) {
+				flog(LOG_ERR, "unable to redirect stdout to /dev/null");
+				exit(-1);
+			}
+			if (open("/dev/null", O_RDWR) == -1) {
+				flog(LOG_ERR, "unable to redirect stderr to /dev/null");
+				exit(-1);
+			}
+		}
+	} else {
+		/* Parent.  Make sure the pid file is written before exiting. */
+		close(pipe_ends[1]);
+		pid_t msg = -1;
+		ssize_t rc = read(pipe_ends[0], &msg, sizeof(msg));
+		if (rc != sizeof(msg)) {
+			flog(LOG_ERR, "child failed to signal pid file written: %s", strerror(errno));
+			exit(-1);
+		} else if (msg != pid) {
+			flog(LOG_ERR, "child wrote wrong pid to pid file: %d", msg);
+			exit(-1);
+		} else {
+			dlog(LOG_DEBUG, 5, "child signaled pid file written: %d", msg);
+		}
+		close(pipe_ends[0]);
+
+		return pid;
+	}
+	return 0;
+}
 
 int main(int argc, char *argv[])
 {
@@ -217,15 +299,6 @@ int main(int argc, char *argv[])
 
 	if (!configtest) {
 		flog(LOG_INFO, "version %s started", VERSION);
-
-		/* Calling privsep here, before opening the socket and reading the config
-		 * file, ensures we're not going to be wasting resources in the privsep
-		 * process. */
-		dlog(LOG_DEBUG, 3, "Initializing privsep");
-		if (privsep_init(username, chrootdir) < 0) {
-			flog(LOG_INFO, "Failed to initialize privsep.");
-			exit(1);
-		}
 	}
 
 	/* check that 'other' cannot write the file
@@ -233,7 +306,7 @@ int main(int argc, char *argv[])
 	 */
 	if (check_conffile_perm(username, conf_path) != 0) {
 		if (get_debuglevel() == 0) {
-			flog(LOG_ERR, "Exiting, permissions on conf_file invalid.");
+			flog(LOG_ERR, "exiting, permissions on conf_file invalid");
 			exit(1);
 		} else
 			flog(LOG_WARNING, "Insecure file permissions, but continuing anyway");
@@ -242,7 +315,7 @@ int main(int argc, char *argv[])
 	/* parse config file */
 	struct Interface *ifaces = NULL;
 	if ((ifaces = readin_config(conf_path)) == 0) {
-		flog(LOG_ERR, "Exiting, failed to read config file.");
+		flog(LOG_ERR, "exiting, failed to read config file");
 		exit(1);
 	}
 
@@ -260,23 +333,83 @@ int main(int argc, char *argv[])
 
 	/* if we know how to do it, check whether forwarding is enabled */
 	if (check_ip6_forwarding()) {
-		flog(LOG_WARNING, "IPv6 forwarding seems to be disabled, but continuing anyway.");
+		flog(LOG_WARNING, "IPv6 forwarding seems to be disabled, but continuing anyway");
 	}
 
-	int const pid_fd = open_and_lock_pid_file(daemon_pid_file_ident);
+	int const pidfd = open_and_lock_pid_file(daemon_pid_file_ident);
 
 	/*
 	 * okay, config file is read in, socket and stuff is setup, so
 	 * lets fork now...
 	 */
 	if (daemonize) {
-		do_daemonize(log_method);
+		pid_t pid = do_daemonize(log_method, daemon_pid_file_ident);
+		if (pid != 0 && pid != -1) {
+			/* We want to see clean output from valgrind, so free username, chrootdir,
+			 * and ifaces in the child process. */
+			if (ifaces)
+				free_ifaces(ifaces);
+
+			if (username)
+				free(username);
+
+			if (chrootdir)
+				free(chrootdir);
+
+			exit(0);
+		}
+	} else {
+		if (0 != write_pid_file(daemon_pid_file_ident, getpid())) {
+			flog(LOG_ERR, "failure writing pid file detected");
+			exit(-1);
+		}
 	}
 
-	if (0 != write_pid_file(pid_fd)) {
-		flog(LOG_ERR, "Unable to write PID to %s", daemon_pid_file_ident);
-		exit(-1);
+	check_pid_file(daemon_pid_file_ident);
+
+#ifdef __linux__
+	/* for privsep */ {
+		dlog(LOG_DEBUG, 3, "initializing privsep");
+
+		int pipefds[2];
+
+		if (pipe(pipefds) != 0) {
+			flog(LOG_ERR, "Couldn't create privsep pipe.");
+			return -1;
+		}
+
+		pid_t pid = fork();
+
+		if (pid == -1) {
+			flog(LOG_ERR, "Couldn't fork for privsep.");
+			return -1;
+		}
+
+		if (pid == 0) {
+			/* We want to see clean output from valgrind, so free username, chrootdir,
+			 * and ifaces in the child process. */
+			if (ifaces)
+				free_ifaces(ifaces);
+
+			if (username)
+				free(username);
+
+			if (chrootdir)
+				free(chrootdir);
+
+			close(pipefds[1]);
+
+			privsep_init(pipefds[0]);
+			_exit(0);
+		}
+
+		dlog(LOG_DEBUG, 3, "radvd privsep PID is %d", pid);
+
+		/* Continue execution (will drop privileges soon) */
+		close(pipefds[0]);
+		privsep_set_write_fd(pipefds[1]);
 	}
+#endif
 
 	if (username) {
 		if (drop_root_privileges(username) < 0) {
@@ -288,12 +421,12 @@ int main(int argc, char *argv[])
 	}
 
 	setup_ifaces(sock, ifaces);
-	main_loop(sock, ifaces, conf_path);
+	ifaces = main_loop(sock, ifaces, conf_path);
 	stop_adverts(sock, ifaces);
 
 	flog(LOG_INFO, "removing %s", daemon_pid_file_ident);
 	unlink(daemon_pid_file_ident);
-	close(pid_fd);
+	close(pidfd);
 
 	if (ifaces)
 		free_ifaces(ifaces);
@@ -310,7 +443,7 @@ int main(int argc, char *argv[])
 	return 0;
 }
 
-static void main_loop(int sock, struct Interface *ifaces, char const *conf_path)
+static struct Interface * main_loop(int sock, struct Interface *ifaces, char const *conf_path)
 {
 	struct pollfd fds[2];
 	sigset_t sigmask;
@@ -368,13 +501,16 @@ static void main_loop(int sock, struct Interface *ifaces, char const *conf_path)
 			ts.tv_sec = timeout / 1000;
 			ts.tv_nsec = (timeout - 1000 * ts.tv_sec) * 1000000;
 			tsp = &ts;
-			dlog(LOG_DEBUG, 1, "polling for %g second(s). Next iface is %s.", timeout / 1000.0,
+			dlog(LOG_DEBUG, 1, "polling for %g second(s), next iface is %s", timeout / 1000.0,
 			     next_iface_to_expire->props.name);
 		} else {
-			dlog(LOG_DEBUG, 1, "No iface is next. Polling indefinitely.");
+			dlog(LOG_DEBUG, 1, "no iface is next. Polling indefinitely");
 		}
-
+#ifdef HAVE_PPOLL
 		int rc = ppoll(fds, sizeof(fds) / sizeof(fds[0]), tsp, &sigempty);
+#else
+		int rc = poll(fds, sizeof(fds) / sizeof(fds[0]), 1000*tsp->tv_sec);
+#endif
 
 		if (rc > 0) {
 #ifdef HAVE_NETLINK
@@ -398,7 +534,7 @@ static void main_loop(int sock, struct Interface *ifaces, char const *conf_path)
 				if (len > 0 && pkt_info) {
 					process(sock, ifaces, msg, len, &rcv_addr, pkt_info, hoplimit);
 				} else if (!pkt_info) {
-					dlog(LOG_INFO, 4, "recv_rs_ra returned null pkt_info.");
+					dlog(LOG_INFO, 4, "recv_rs_ra returned null pkt_info");
 				} else if (len <= 0) {
 					dlog(LOG_INFO, 4, "recv_rs_ra returned len <= 0: %d", len);
 				}
@@ -411,89 +547,146 @@ static void main_loop(int sock, struct Interface *ifaces, char const *conf_path)
 		}
 
 		if (sigint_received) {
-			flog(LOG_WARNING, "Exiting, %d sigint(s) received.", sigint_received);
+			flog(LOG_WARNING, "exiting, %d sigint(s) received", sigint_received);
 			break;
 		}
 
 		if (sigterm_received) {
-			flog(LOG_WARNING, "Exiting, %d sigterm(s) received.", sigterm_received);
+			flog(LOG_WARNING, "exiting, %d sigterm(s) received", sigterm_received);
 			break;
 		}
 
 		if (sighup_received) {
-			dlog(LOG_INFO, 3, "sig hup received.");
+			dlog(LOG_INFO, 3, "sig hup received");
 			ifaces = reload_config(sock, ifaces, conf_path);
 			sighup_received = 0;
 		}
 
 		if (sigusr1_received) {
-			dlog(LOG_INFO, 3, "sig usr1 received.");
+			dlog(LOG_INFO, 3, "sig usr1 received");
 			reset_prefix_lifetimes(ifaces);
 			sigusr1_received = 0;
 		}
 
 	}
+
+	return ifaces;
 }
 
-static void do_daemonize(int log_method)
+static pid_t do_daemonize(int log_method, char const * daemon_pid_file_ident)
 {
-	int rc = -1;
+	pid_t pid = -1;
 
 	if (L_STDERR_SYSLOG == log_method || L_STDERR == log_method) {
-		rc = daemon(1, 1);
+		pid = daemonp(1, 1, daemon_pid_file_ident);
 	} else {
-		rc = daemon(0, 0);
+		pid = daemonp(0, 0, daemon_pid_file_ident);
 	}
 
-	if (-1 == rc) {
-		flog(LOG_ERR, "Unable to daemonize: %s", strerror(errno));
+	if (-1 == pid) {
+		flog(LOG_ERR, "unable to daemonize: %s", strerror(errno));
 		exit(-1);
 	}
+	return pid;
+}
+
+static int open_pid_file(char const * daemon_pid_file_ident)
+{
+	int pidfd = open(daemon_pid_file_ident, O_SYNC | O_CREAT | O_RDWR, 0644);
+	if (-1 == pidfd) {
+		flog(LOG_ERR, "unable to open pid file, %s: %s", daemon_pid_file_ident, strerror(errno));
+		exit(-1);
+	} else {
+		dlog(LOG_DEBUG, 5, "opened pid file %s", daemon_pid_file_ident);
+	}
+	return pidfd;
 }
 
 static int open_and_lock_pid_file(char const * daemon_pid_file_ident)
 {
 	dlog(LOG_DEBUG, 3, "radvd startup PID is %d", getpid());
 
-	int pid_fd = open(daemon_pid_file_ident, O_CREAT | O_RDWR, 0644);
-	if (-1 == pid_fd) {
-		flog(LOG_ERR, "Unable to open pid file, %s: %s", daemon_pid_file_ident, strerror(errno));
-		exit(-1);
-	} else {
-		dlog(LOG_DEBUG, 4, "opened pid file %s", daemon_pid_file_ident);
-	}
+	int pidfd = open_pid_file(daemon_pid_file_ident);
 
-	int lock = flock(pid_fd, LOCK_EX | LOCK_NB);
+	int lock = flock(pidfd, LOCK_EX | LOCK_NB);
 	if (0 != lock) {
-		flog(LOG_ERR, "Unable to lock pid file, %s: %s", daemon_pid_file_ident, strerror(errno));
+		flog(LOG_ERR, "unable to lock pid file, %s: %s", daemon_pid_file_ident, strerror(errno));
 		exit(-1);
 	} else {
 		dlog(LOG_DEBUG, 4, "locked pid file %s", daemon_pid_file_ident);
 	}
 
-	return pid_fd;
+	return pidfd;
 }
 
-static int write_pid_file(int pid_fd)
+static int write_pid_file(char const * daemon_pid_file_ident, pid_t pid)
 {
+	int pidfd = open_pid_file(daemon_pid_file_ident);
 	char pid_str[20] = {""};
-	sprintf(pid_str, "%d", getpid());
+	sprintf(pid_str, "%d", pid);
 	dlog(LOG_DEBUG, 3, "radvd PID is %s", pid_str);
 	size_t len = strlen(pid_str);
-	int rc = write(pid_fd, pid_str, len);
+	int rc = write(pidfd, pid_str, len);
 	if (rc != (int)len) {
 		return -1;
 	}
 	char newline[] = {"\n"};
 	len = strlen(newline);
-	rc = write(pid_fd, newline, len);
+	rc = write(pidfd, newline, len);
 	if (rc != (int)len) {
 		return -1;
 	}
-	return 0;
-
+	rc = fsync(pidfd);
+	if (rc != 0) {
+		dlog(LOG_DEBUG, 4, "failed to fsync pid file: %s", daemon_pid_file_ident);
+	}
+	rc = close(pidfd);
+	if (rc != 0) {
+		dlog(LOG_DEBUG, 4, "failed to close pid file: %s", daemon_pid_file_ident);
+	}
+	char * dirstrcopy = strdup(daemon_pid_file_ident);
+	char * dirstr = dirname(dirstrcopy);
+	int dirfd = open(dirstr, O_RDONLY);
+	rc = fsync(dirfd);
+	if (rc != 0) {
+		dlog(LOG_DEBUG, 4, "failed to fsync pid dir: %s", dirstr);
+	}
+	rc = close(dirfd);
+	if (rc != 0) {
+		dlog(LOG_DEBUG, 4, "failed to close pid dir: %s", dirstr);
+	}
+	free(dirstrcopy);
+	dlog(LOG_DEBUG, 4, "wrote pid %d to pid file: %s", pid, daemon_pid_file_ident);
+	return rc;
 }
 
+static void check_pid_file(char const * daemon_pid_file_ident)
+{
+	FILE * pidfile = fopen(daemon_pid_file_ident, "r");
+
+	if (!pidfile) {
+		flog(LOG_ERR, "unable to open pid file, %s: %s", daemon_pid_file_ident, strerror(errno));
+		exit(-1);
+	}
+
+	pid_t pid = -1;
+
+	int rc = fscanf(pidfile, "%d", &pid);
+	fclose(pidfile);
+
+	if (rc != 1) {
+		flog(LOG_ERR, "unable to read pid from pid file: %s", daemon_pid_file_ident);
+		exit(-1);
+	}
+
+	if (pid != getpid()) {
+		flog(LOG_ERR, "pid in file, %s, doesn't match getpid(): %d != %d", daemon_pid_file_ident, pid, getpid());
+		exit(-1);
+	}
+	dlog(LOG_DEBUG, 4, "validated pid file, %s: %d", daemon_pid_file_ident, pid);
+}
+
+
 static void timer_handler(int sock, struct Interface *iface)
 {
 	dlog(LOG_DEBUG, 1, "timer_handler called for %s", iface->props.name);
@@ -544,7 +737,7 @@ static void stop_advert_foo(struct Interface *iface, void *data)
 {
 	if (!iface->UnicastOnly) {
 		/* send a final advertisement with zero Router Lifetime */
-		dlog(LOG_DEBUG, 4, "stopping all adverts on %s.", iface->props.name);
+		dlog(LOG_DEBUG, 4, "stopping all adverts on %s", iface->props.name);
 		iface->state_info.cease_adv = 1;
 		int sock = *(int *)data;
 		send_ra_forall(sock, iface, NULL);
@@ -593,7 +786,7 @@ static struct Interface *reload_config(int sock, struct Interface *ifaces, char
 	/* reread config file */
 	ifaces = readin_config(conf_path);
 	if (!ifaces) {
-		flog(LOG_ERR, "Exiting, failed to read config file.");
+		flog(LOG_ERR, "exiting, failed to read config file");
 		exit(1);
 	}
 	setup_ifaces(sock, ifaces);
